//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     rlits.cxx
// Purpose:    List of literals in a rule
// Last Change:02.10.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Interface:
//=============================================================================

#include "ver.h"
#include "out.h"
#include "rvars.h"
#include "rlit.h"

#include "rlits.h"

//=============================================================================
// Exported Functions:
//=============================================================================

//-----------------------------------------------------------------------------
// Constructor from list element:
//-----------------------------------------------------------------------------

rlits_c::rlits_c(rlit_t head)
{
	// Check Parameters:
	CHECK(head != 0, "rlits_c::Constructor 1: Head is NIL");
	CHECK(head->valid(), "rlits_c::Constructor 1: Head is invalid");

	// Initialize Magic Number:
#ifdef VER_DEBUG
	Magic = RLITS_MAGIC;
#endif

	// Initialize object:
	Head = head;
	Tail = 0;
}

//-----------------------------------------------------------------------------
// Constructor from list of input formula literals:
//-----------------------------------------------------------------------------

rlits_c::rlits_c(lit_t *lits, rvars_t vars, bool_t binding)
{
	rlit_t	rlit;
	rlits_t	previous, next;

	// Check Parameters:
	CHECK(lits != 0, "rlits_c::Constructor 2: lits is NIL");
	CHECK(*lits != 0, "rlits_c::Constructor 2: lits is empty");
	CHECK(vars != 0, "rlits_c::Constructor 2: vars is NIL");
	CHECK(vars->valid(), "rlits_c::Constructor 2: vars is invalid");
	CHECK(binding==BOOL_TRUE || binding==BOOL_FALSE,
			"rlits_c::Constructor 2: binding is invalid");

	// Initialize Magic Number:
#ifdef VER_DEBUG
	Magic = RLITS_MAGIC;
#endif

	// Initialize object by translating first literal:
	CHECK((*lits)->valid(), "rlits_c: Constructor 2: first lit is invalid");
	Head = new rlit_c((*lits), vars, binding);
	Tail = 0;
	previous = this;

	// Loop over remaining input literals:
	while(*++lits) {
		CHECK((*lits)->valid(),
			"rlits_c: Constructor 2: input literal is invalid");
		rlit = new rlit_c((*lits), vars, binding);
		next = new rlits_c(rlit);
		previous->append(next);
		previous = next;
	}
}

//-----------------------------------------------------------------------------
// Destructor:
//-----------------------------------------------------------------------------

rlits_c::~rlits_c(void)
{
#ifdef	VER_DEBUG
	Magic = 0;
#endif
}

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
bool_t	rlits_c::valid(void) const
{
	// Check object pointer:
	CHECK(this != 0, "rlits_c::valid: NIL object");

	// Magic number must be correct:
	if(Magic != RLITS_MAGIC)
		return(BOOL_FALSE);

	// Head must be not null:
	if(Head == 0)
		return(BOOL_FALSE);

	// Head must be valid:
	if(Head->valid() != BOOL_TRUE)
		return(BOOL_FALSE);

	// Ok:
	return(BOOL_TRUE);
}
#endif

//-----------------------------------------------------------------------------
// Print this term list:
//-----------------------------------------------------------------------------

void rlits_c::print(rvars_t vars, str_t separator) const
{
	rlits_c *p;

	// Check Consistency:
	CHECK(valid(), "rlits_c::print: Bad object");

	// Print list iteratively:
	Head->print(vars);
	p = Tail;
	while(p != 0) {
		out_c::print_str(separator);
		p->get_head()->print(vars);
		p = p->get_tail();
	}
}

